#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from cinder import service
from cinder import test
import mock
import os
from oslo_config import cfg
from oslo_service import sslutils

test_service_opts = [
    cfg.BoolOpt('test_osapi_volume_use_ssl',
                default=False,
                help='Wraps the socket in a SSL context if True is set.'
                'A certificate file and key file must be specified.'),
    cfg.StrOpt('test_cert_file',
               help="Certificate file to use when starting "
                    "the server securely.",
               deprecated_group='DEFAULT',
               deprecated_name='ssl_cert_file'),
    cfg.StrOpt('test_key_file',
               help="Private key file to use when starting "
                    "the server securely.",
               deprecated_group='DEFAULT',
               deprecated_name='ssl_key_file'), ]

ssl_opts = [
    cfg.StrOpt('ca_file',
               help="CA certificate file to use to verify "
                    "connecting clients.",
               deprecated_group='DEFAULT',
               deprecated_name='ssl_ca_file'),
    cfg.StrOpt('cert_file',
               help="Certificate file to use when starting "
                    "the server securely.",
               deprecated_group='DEFAULT',
               deprecated_name='ssl_cert_file'),
    cfg.StrOpt('key_file',
               help="Private key file to use when starting "
                    "the server securely.",
               deprecated_group='DEFAULT',
               deprecated_name='ssl_key_file'),
    cfg.StrOpt('version',
               help='SSL version to use (valid only if SSL enabled). '
                    'Valid values are TLSv1 and SSLv23. SSLv2, SSLv3, '
                    'TLSv1_1, and TLSv1_2 may be available on some '
                    'distributions.'
               ),
    cfg.StrOpt('ciphers',
               help='Sets the list of available ciphers. value should be a '
                    'string in the OpenSSL cipher list format.'
               ),
]

CONF = cfg.CONF
CONF.register_opts(test_service_opts)
CONF.register_opts(ssl_opts, 'ssl')

SSL_CERT_DIR = os.path.normpath(os.path.join(
    os.path.dirname(os.path.abspath(__file__)),
    'ssl_cert'))

CERT_FILE = os.path.join(SSL_CERT_DIR, 'certificate.crt')
KEY_FILE = os.path.join(SSL_CERT_DIR, 'privatekey.key')


class TestWSGIService(test.TestCase):

    def setUp(self):
        super(TestWSGIService, self).setUp()

    @mock.patch('oslo_service.wsgi.Loader')
    def test_ssl_default(self, mock_loader):
        self.override_config('osapi_volume_listen_port', 0)  # random port
        self.override_config('osapi_volume_use_ssl',
                             CONF.test_osapi_volume_use_ssl)
        self.override_config("cert_file", CONF.test_cert_file, 'ssl')
        self.override_config('key_file', CONF.test_key_file, 'ssl')

        # Test WSGI service
        test_service = service.WSGIService("osapi_volume")

        # use_ssl should be false when using default settings
        self.assertEqual(test_service.use_ssl,
                         CONF.test_osapi_volume_use_ssl)  # false by default

        test_service.stop()
        test_service.wait()  # wait for service to stop

        self.assertTrue(mock_loader.called)

    @mock.patch('os.path.exists')
    @mock.patch('oslo_service.wsgi.Loader')
    def test_ssl_enabled_no_cert_no_key(self, mock_loader, mock_path_check):
        mock_path_check.side_effect = [False, False, False]
        # Override ssl enabler without certificate and private key
        self.override_config('osapi_volume_listen_port', 0)  # random port
        self.override_config('osapi_volume_use_ssl', True)

        """ sslutils.is_enabled will determine whether or not to wrap the
            socket in ssl
            sslutils.is_enabled should raise an exception when either no cert
            file or no key file, but not when both (runs without ssl)
        """

        # self.assertRaises(RuntimeError, sslutils.is_enabled, CONF)

        # Now test WSGI service
        try:
            test_service = service.WSGIService("osapi_volume")
        except RuntimeError:
            print('!!!!!!!! Should not have failed !!!!!!!!')  # noqa
            pass
        else:
            test_service.stop()
            test_service.wait()  # wait for service to stop
            print('Runs without SSL. No key, no cert')  # noqa

        self.assertTrue(mock_loader.called)

    @mock.patch('os.path.exists')
    @mock.patch('oslo_service.wsgi.Loader')
    def test_ssl_enabled_has_cert_no_key(self, mock_loader, mock_path_check):
        mock_path_check.side_effect = [True, False, False]
        # Override ssl enabler without private key
        self.override_config('osapi_volume_listen_port', 0)  # random port
        self.override_config('osapi_volume_use_ssl', True)
        self.override_config("cert_file", CERT_FILE, 'ssl')
        self.override_config('key_file', '', 'ssl')

        """ sslutils.is_enabled will determine whether or not to wrap the
            socket in ssl
            sslutils.is_enabled should raise an exception when either no cert
            file or no key file
        """

        self.assertRaises(RuntimeError, sslutils.is_enabled, CONF)

        # Now test WSGI service
        try:
            test_service = service.WSGIService("osapi_volume")
        except RuntimeError:
            print('!!!!!!!! Failed as it should have !!!!!!!!')  # noqa
            pass
        else:
            test_service.stop()
            test_service.wait()  # wait for service to stop

        self.assertTrue(mock_loader.called)

    @mock.patch('os.path.exists')
    @mock.patch('oslo_service.wsgi.Loader')
    def test_ssl_enabled_no_cert_has_key(self, mock_loader, mock_path_check):
        mock_path_check.side_effect = [True, False, False]
        # Override ssl enabler without certificate
        self.override_config('osapi_volume_listen_port', 0)  # random port
        self.override_config('osapi_volume_use_ssl', True)
        self.override_config("cert_file", '', 'ssl')
        self.override_config('key_file', KEY_FILE, 'ssl')

        """ sslutils.is_enabled will determine whether or not to wrap the
            socket in ssl
            sslutils.is_enabled should raise an exception when either no cert
            file or no key file
        """

        self.assertRaises(RuntimeError, sslutils.is_enabled, CONF)

        # Now test WSGI service
        try:
            test_service = service.WSGIService("osapi_volume")
        except RuntimeError:
            print('!!!!!!!! Failed as it should have !!!!!!!!')  # noqa
            pass
        else:
            test_service.stop()
            test_service.wait()  # wait for service to stop

        self.assertTrue(mock_loader.called)

    @mock.patch('os.path.exists')
    @mock.patch('oslo_service.wsgi.Loader')
    def test_ssl_enabled_has_cert_and_key(self, mock_loader, mock_path_check):
        # Override ssl enabler with cert and key
        self.override_config('osapi_volume_listen_port', 0)  # random port
        self.override_config('osapi_volume_use_ssl', True)
        self.override_config("cert_file", CERT_FILE, 'ssl')
        self.override_config('key_file', KEY_FILE, 'ssl')

        """ sslutils.is_enabled will determine whether or not to wrap the
            socket in ssl
            sslutils.is_enabled should raise an exception when either no cert
            file or no key file
        """

        # self.assert(RuntimeError, sslutils.is_enabled, CONF)

        # Now test WSGI service
        try:
            test_service = service.WSGIService("osapi_volume")
        except RuntimeError:
            print('!!!!!!!! Should have not failed !!!!!!!!')  # noqa
            raise

        test_service.stop()
        test_service.wait()  # wait for service to stop

        self.assertTrue(mock_loader.called)
